SystemOrganization addCategory: #'Zinc-Seaside'!

WAServerAdaptor subclass: #ZnZincServerAdaptor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Seaside'!

!ZnZincServerAdaptor commentStamp: '<historical>' prior: 0!
I convert between Seaside and Zinc HTTP requests and responses.

Instance Variables:
	server	<ZnServer>!

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/9/2010 20:48'!
basicStart
	(server := ZnServer on: self port)
		delegate: [ :request | self process: request ];
		start! !

!ZnZincServerAdaptor methodsFor: 'private' stamp: 'pmm 9/8/2010 16:50'!
basicStop
	server isNil
		ifFalse: [ server stop ].
	server := nil! !

!ZnZincServerAdaptor methodsFor: 'defaults' stamp: 'pmm 9/8/2010 16:54'!
defaultCodec
	^ GRCodec forEncoding: 'utf-8'! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:37'!
isRunning
	^ server notNil and: [ server isRunning ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:51'!
isStarting
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/9/2010 22:07'!
isStopped
	^ server isNil or: [ server isRunning not ]! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/8/2010 16:52'!
isStopping
	^ false ! !

!ZnZincServerAdaptor methodsFor: 'testing' stamp: 'pmm 9/9/2010 22:16'!
isUrlEncoded: aZincRequest
	"answers whether the body of the given request is url encoded"
	| contentType |
	contentType := aZincRequest headers at: 'Content-Type' ifAbsent: [ nil ].
	^ contentType = 'application/x-www-form-urlencoded'! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/9/2010 22:08'!
requestAddressFor: aZincRequest
	^ aZincRequest  headers
		at: ZnConstants remoteAddressHeader 
		ifAbsent: [ nil ]! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 9/9/2010 20:25'!
requestBodyFor: aZincRequest
	^ aZincRequest entity! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/9/2010 22:11'!
requestCookiesFor: aZincRequest
	| cookieString |
	cookieString := aZincRequest headers
		at: 'Cookie' ifAbsent: [ ^#() ].
	^  WARequestCookie fromString: cookieString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/9/2010 22:23'!
requestFieldsFor: aZincRequest
	| fields |
	fields := WARequestFields new.
	aZincRequest method = #POST ifTrue: [ 
		(self isUrlEncoded: aZincRequest) ifTrue: [
			| url entity body |
			entity := aZincRequest entity.
			(entity isKindOf: ZnByteArrayEntity) ifTrue: [
				body := entity bytes asString ].
			(entity isKindOf: ZnStringEntity) ifTrue: [
				body := entity string ].
			url := (WAUrl absolute: '/?', body)
				decodedWith: self codec.
			fields addAll: url queryFields ] ].
	^ fields
	! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 9/9/2010 20:02'!
requestHeadersFor: aZincRequest
	| fields |
	fields := Dictionary new.
	aZincRequest headersDo: [ :key :value |
		fields at: key asLowercase put: value ].
	^fields! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:15'!
requestMethodFor: aZincRequest
	^ aZincRequest method greaseString! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:18'!
requestUrlFor: aZincRequest
	^ (WAUrl absolute: aZincRequest uri asString)
		decodedWith: self codec.! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 17:19'!
requestVersionFor: aZincRequest
	^ WAHttpVersion fromString: aZincRequest requestLine version! !

!ZnZincServerAdaptor methodsFor: 'converting' stamp: 'pmm 9/8/2010 18:00'!
responseFrom: aRequestContext
	| partialHeaders cookies fullHeaders seasideResponse contents entity contentType |
	
	seasideResponse := aRequestContext response.
	partialHeaders := seasideResponse headers.
	cookies := seasideResponse cookies.
	fullHeaders := ZnHeaders defaultResponseHeaders.
	partialHeaders keysAndValuesDo: [ :key :value |
		fullHeaders at: key put: value ].
	cookies do: [ :each |
		fullHeaders at: 'Set-Cookie' put: each oldNetscapeString.
		fullHeaders at: 'Set-Cookie2' put: each rfc2965String ].
	contentType := seasideResponse contentType greaseString.
	contents := seasideResponse contents.
	entity := (contents isString
		ifTrue: [ (ZnStringEntity type:  contentType length: contents size)
			string: contents;
			yourself ]
		ifFalse: [ (ZnByteArrayEntity type: contentType length: contents size)
			bytes: contents;
			yourself ]) .
	^ ZnResponse new
		statusLine: (ZnStatusLine code: seasideResponse status);
		headers: fullHeaders;
		entity: entity;
		yourself! !

!ZnZincServerAdaptor methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:38'!
statusString
	^ 'zinc on port ', self port greaseString, 
		( self isRunning ifTrue: [ ' [running]' ] ifFalse: [ ' [stopped]' ] )! !
